@@ -19,6 +19,20 @@
 /// root2.insert(1);
 /// root2.insert(6);
 /// ```
+///
+/// It also supports both consumable and non-cosumable iterator
+/// which returns values inorder.
+///
+/// ```rust
+/// use ds_bst::BinarySearchTree;
+/// let root = BinarySearchTree::from(vec![1,2,3,4,5,6,7,8,9]);
+/// for value in &root {
+///     // It will print values in-order traversal
+///     println!("{}", *value);
+/// }
+/// ```
+use std::cmp::{max};
+
 pub struct BinarySearchTree<T> {
     val: T,
     left: Option<Box<BinarySearchTree<T>>>,
@@ -67,7 +81,6 @@ impl<T: PartialOrd + Copy> BinarySearchTree<T> {
 
     /// Inorder traverse tree which yields elements in sorted order.
     /// Uses `O(n)` time.
-    /// TODO: Implement Iterator
     pub fn inorder(&self) -> Vec<T> {
         let mut ret: Vec<T> = Vec::new();
 
@@ -91,7 +104,6 @@ impl<T: PartialOrd + Copy> BinarySearchTree<T> {
 
     /// Traverse tree in preorder.
     /// Uses `O(n)` time.
-    /// TODO: Implement using iterator
     pub fn preorder(&self) -> Vec<T> {
         let mut ret: Vec<T> = Vec::new();
 
@@ -113,6 +125,26 @@ impl<T: PartialOrd + Copy> BinarySearchTree<T> {
         ret
     }
 
+    /// Calculates tree maximum height
+    /// Worst case O(n)
+    pub fn height(&self) -> usize {
+        let hl: usize = match self.left {
+            None => { 0 },
+            Some(ref node) => {
+                node.height()
+            }
+        };
+
+        let hr: usize = match self.right{
+            None => { 0 },
+            Some(ref node) => {
+                node.height()
+            }
+        };
+
+        max(hl, hr) + 1
+    }
+
     /// Inserts an element in a tree.
     /// Uses `O(n)` time.
     pub fn insert(&mut self, val: T) {
@@ -170,6 +202,82 @@ impl<T: PartialOrd + Copy> BinarySearchTree<T> {
     }
 }
 
+/// BinarySearchTreeIterator
+pub struct BinarySearchTreeIter<'a, T> {
+    nodes: Vec<&'a T>
+}
+
+impl<'a, T> BinarySearchTreeIter<'a, T>
+    where
+        T: PartialOrd + Copy
+{
+    /// Construct nodes based on input tree. By default
+    /// it uses in-order traversal for iterator.
+    fn new(root: &'a BinarySearchTree<T>) -> Self {
+        let mut iter = BinarySearchTreeIter {
+            nodes: Vec::new()
+        };
+
+        iter.inorder(root);
+
+        iter
+    }
+
+    /// In-order tree traversal
+    fn inorder(&mut self, tree: &'a BinarySearchTree<T>) {
+        match tree.right {
+            None => {},
+            Some(ref node) => {
+                self.inorder(node);
+            }
+        };
+        self.nodes.push(&tree.val);
+        match tree.left {
+            None => {},
+            Some(ref node) => {
+                self.inorder(node);
+            }
+        }
+    }
+}
+
+/// Implement iterator for BinarySearchTreeIter
+/// nodes are stored in flat array. It just pop outs node
+impl<'a, T> Iterator for BinarySearchTreeIter<'a, T>
+    where
+        T: PartialOrd + Copy,
+{
+    type Item = &'a T;
+
+    fn next(&mut self) -> Option<Self::Item> {
+        self.nodes.pop()
+    }
+}
+
+/// implement consumable IntoIterator for BinarySearchTree
+impl<T> IntoIterator for BinarySearchTree<T>
+    where
+        T: PartialOrd + Copy,
+{
+    type Item = T;
+    type IntoIter = std::vec::IntoIter<Self::Item>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        self.inorder().into_iter()
+    }
+}
+
+/// Implement non-consumable IntoIterator for BinarySearchTree
+impl<'a, T> IntoIterator for &'a BinarySearchTree<T>
+    where
+        T: PartialOrd + Copy {
+    type Item = &'a T;
+    type IntoIter = BinarySearchTreeIter<'a, T>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        BinarySearchTreeIter::new(self)
+    }
+}
 
 
 #[cfg(test)]
@@ -217,4 +325,35 @@ mod tests {
         assert_eq!(root.exists(1.8), true);
         assert_eq!(root.find_max(), 1.9);
     }
+    #[test]
+    fn iterator_consumable() {
+        let root = BinarySearchTree::from(vec![1,2,3]);
+        let mut i = 1;
+
+        for v in root {
+            assert_eq!(v, i);
+            i = i + 1;
+        }
+        // root is now consumed and cannot be used here
+    }
+    #[test]
+    fn iterator_non_consumable() {
+        let root = BinarySearchTree::from(vec![1,2,3]);
+        let mut i = 1;
+        for v in &root {
+            assert_eq!(*v, i);
+            i = i + 1;
+        };
+
+        assert_eq!(root.find_max(), 3);
+        assert_eq!(root.height(), 2);
+    }
+    #[test]
+    fn height() {
+        let root = BinarySearchTree::from(vec![1]);
+        assert_eq!(root.height(), 1);
+
+        let root2 = BinarySearchTree::from(vec![11,20,29,32,41,65,50,91,72,99]);
+        assert_eq!(root2.height(), 4)
+    }
 }
